@@ -51,17 +51,32 @@ class DerivedDataApi(object):
                 print(e)
                 return pd.DataFrame([])
 
-    def get_index_valuation_develop(self, index_id, start_date):
+    def get_index_valuation_develop(self, index_ids, start_date, end_date):
         with DerivedDatabaseConnector().managed_session() as mn_session:
             try:
                 query = mn_session.query(
                     IndexValuationDevelop
                 ).filter(
-                    IndexValuationDevelop.index_id == index_id,
+                    IndexValuationDevelop.index_id.in_(index_ids),
                     IndexValuationDevelop.datetime >= start_date,
+                    IndexValuationDevelop.datetime <= end_date,
                 ).order_by(IndexValuationDevelop.datetime.asc())
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
                 print(e)
                 return pd.DataFrame([])
+
+    def get_asset_allocation_info(self, version:int=2):
+        with DerivedDatabaseConnector().managed_session() as mn_session:
+            try:
+                query = mn_session.query(
+                    AssetAllocationInfo
+                ).filter(
+                    AssetAllocationInfo.version == version
+                ).order_by(AssetAllocationInfo.allocation_id)
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print(e)
+                return pd.DataFrame([])
\ No newline at end of file
