@@ -12,10 +12,11 @@ import os
 import csv
 import traceback
 from sqlalchemy import distinct
+from sqlalchemy.orm import sessionmaker
 from ...wrapper.mysql import RawDatabaseConnector
 from ...view.raw_models import *
 from ...api.basic import BasicDataApi
-
+from ...api.raw import RawDataApi
 
 class EmRawDataDownloader(object):
 
@@ -31,10 +32,15 @@ class EmRawDataDownloader(object):
 
         self._scale_change_dates = ['0331', '0630', '0930', '1231']
 
-    def _upload_raw(self, df: pd.DataFrame, table_name: str, if_exists='append'):
+        self.updated_count = {}
+
+    def _upload_raw(self, df: pd.DataFrame, table_name: str):
         print(table_name)
-        df.to_sql(table_name, RawDatabaseConnector().get_engine(),
-                  index=False, if_exists=if_exists)
+        df.to_sql(table_name, RawDatabaseConnector().get_engine(), index=False, if_exists='append')
+
+        if table_name not in self.updated_count:
+            self.updated_count[table_name] = 0
+        self.updated_count[table_name] += df.shape[0]
 
     def _main_callback(self, quantdata):
         '''
@@ -165,33 +171,40 @@ class EmRawDataDownloader(object):
             return False
 
     def em_fund_scale(self, start_date, end_date):
-        # Get all fund ids, 功能函数-板块成分
-        # http://quantapi.eastmoney.com/Cmd/Sector?from=web
-        fund_id_result = c.sector('507013', end_date)
-        if fund_id_result.ErrorCode != 0:
-            print(f'Failed to get fund id list: {fund_id_result.ErrorMsg}')
-            return
+        try:
+            # Get all fund ids, 功能函数-板块成分
+            # http://quantapi.eastmoney.com/Cmd/Sector?from=web
+            fund_id_result = c.sector('507013', end_date)
+            if fund_id_result.ErrorCode != 0:
+                print(f'Failed to get fund id list: {fund_id_result.ErrorMsg}')
+                return
 
-        fund_id_list = fund_id_result.Data[0::2]
-        fund_id_str = ','.join(fund_id_list)
+            fund_id_list = fund_id_result.Data[0::2]
+            fund_id_str = ','.join(fund_id_list)
 
-        for d in self._get_date_list(start_date, end_date):
-            month_day = d.strftime('%m%d')
-            if month_day not in self._scale_change_dates:
-                continue
-
-            print(d)
-
-            df = c.css(fund_id_str, 'FUNDSCALE',
-                       f'EndDate={end_date},Ispandas=1').reset_index()
-            df['DATES'] = d
-            self._upload_raw(df, EmFundScale.__table__.name)
+            for d in self._get_date_list(start_date, end_date):
+                df = c.css(fund_id_str, 'FUNDSCALE', f'EndDate={d},Ispandas=1').reset_index()
+                df['DATES'] = d
+                self._upload_raw(df, EmFundScale.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False
 
     def em_index_price(self, em_ids, start_date, end_date):
+        start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
+        start_date = start_date.strftime('%Y%m%d')
+        self.delete_em_index_price(em_ids, start_date, end_date)
         try:
-            em_id_str = ','.join(em_ids) if em_ids is list else em_ids
-            df = c.csd(em_id_str, "CLOSE", start_date, end_date,
-                    "period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
+            res = []
+            if isinstance(em_ids, list):
+                for em_id in em_ids:
+                    df = c.csd(em_id, "CLOSE", start_date, end_date,"period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
+                    res.append(df)
+                df = pd.concat(res)
+            else:
+                df = c.csd(em_ids, "CLOSE", start_date, end_date,"period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
 
             df = df.reset_index().rename(columns={
                 'CODES': 'em_id',
@@ -206,6 +219,15 @@ class EmRawDataDownloader(object):
             traceback.print_stack()
             return False
 
+    def delete_em_index_price(self, em_ids, start_date, end_date):
+        try:
+            RawDataApi().delete_em_index_price(em_ids, start_date, end_date)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False    
+
     def em_index(self, start_date, end_date):
         # ----------------------
         # H11073.CSI,信用债
@@ -297,7 +319,8 @@ class EmRawDataDownloader(object):
                 if not isinstance(temp_df, pd.DataFrame) and temp_df.ErrorCode != 0:
                     print(f'failed to get stock price info: {temp_df.ErrorMsg} (date){date}')
                     return
-                df_list.append(temp_df)
+                # 过滤掉个股未上市时的数据，以及日期为nan的行（表示整行为无效数据）
+                df_list.append(temp_df.loc[temp_df['TRADESTATUS'] != '未上市'].loc[~temp_df['DATES'].isna()])
 
             df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockPrice.__getattribute__(EmStockPrice, x).name)
             # 更新到db
@@ -308,6 +331,46 @@ class EmRawDataDownloader(object):
             traceback.print_stack()
             return False
 
+    def em_stock_post_price(self, start_date: str, end_date: str):
+        try:
+            # 获取区间内所有交易日
+            tradedates = c.tradedates(start_date, end_date, "period=1,order=1,market=CNSESH")
+            if tradedates.ErrorCode != 0:
+                print(f'[em_stock_post_price] failed to get trade dates: {tradedates.ErrorMsg}')
+                return
+            if len(tradedates.Data) == 0:
+                print(f'[em_stock_post_price] no trade dates, return immediately (start_date){start_date} (end_date){end_date}')
+                return
+
+            # 获取所有A股股票的ID, 功能函数-板块成分
+            # http://quantapi.eastmoney.com/Cmd/Sector?from=web
+            # 全部A股股票
+            stock_id_result = c.sector('001004', end_date)
+            if stock_id_result.ErrorCode != 0:
+                print(f'failed to get stock id list: {stock_id_result.ErrorMsg}')
+                return
+
+            df_list = []
+            for date in tradedates.Data:
+                # 获取个股股价信息
+                temp_df = c.csd(stock_id_result.Codes,
+                                'OPEN,CLOSE,HIGH,LOW,PRECLOSE,AVERAGE,VOLUME,AMOUNT,TRADESTATUS,TAFACTOR',
+                                date, date, 'period=1,adjustflag=2,curtype=1,order=1,market=0,Ispandas=1')
+                if not isinstance(temp_df, pd.DataFrame) and temp_df.ErrorCode != 0:
+                    print(f'failed to get stock price info: {temp_df.ErrorMsg} (date){date}')
+                    return
+                # 过滤掉个股未上市时的数据，以及日期为nan的行（表示整行为无效数据）
+                df_list.append(temp_df.loc[temp_df['TRADESTATUS'] != '未上市'].loc[~temp_df['DATES'].isna()])
+
+            df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockPostPrice.__getattribute__(EmStockPostPrice, x).name)
+            # 更新到db
+            self._upload_raw(df, EmStockPostPrice.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False
+
     def em_stock_info(self, end_date: str):
         # 获取所有A股股票的ID, 功能函数-板块成分
         # http://quantapi.eastmoney.com/Cmd/Sector?from=web
@@ -359,6 +422,31 @@ class EmRawDataDownloader(object):
             # 更新到db
             self._upload_raw(df, EmStockInfo.__table__.name)
 
+    def add_columns_to_fin_fac(self, end_date: str):
+        # 获取所有A股股票的ID, 功能函数-板块成分
+        # http://quantapi.eastmoney.com/Cmd/Sector?from=web
+        # 全部A股股票
+        stock_id_result = c.sector('001004', end_date)
+        if stock_id_result.ErrorCode != 0:
+            print(f'failed to get stock id list: {stock_id_result.ErrorMsg}')
+            return
+
+        # 获取主营收入构成(按产品)
+        df = c.css(stock_id_result.Codes, "MBSALESCONS", f'ReportDate={end_date},type=2,Ispandas=1')
+        if not isinstance(df, pd.DataFrame) and df.ErrorCode != 0:
+            print(f'failed to get stock financial factors: {df.ErrorMsg}')
+            return
+
+        Session = sessionmaker(RawDatabaseConnector().get_engine())
+        db_session = Session()
+        for row in db_session.query(EmStockFinFac):
+            if row.DATES != datetime.datetime.strptime(end_date, '%Y%m%d').date().year:
+                continue
+            if row.CODES in df.index:
+                row.MBSALESCONS_P = df.at[row.CODES, 'MBSALESCONS']
+        db_session.commit()
+        db_session.close()
+
     def em_stock_fin_fac(self, end_date: str):
         # 获取所有A股股票的ID, 功能函数-板块成分
         # http://quantapi.eastmoney.com/Cmd/Sector?from=web
@@ -372,13 +460,24 @@ class EmRawDataDownloader(object):
         # 获取无参数指标
         df_no_param = c.css(stock_id_result.Codes,
                             'EBIT,EBITDA,EXTRAORDINARY,LOWERDIANDNI,GROSSMARGIN,OPERATEINCOME,INVESTINCOME,EBITDRIVE,TOTALCAPITAL,WORKINGCAPITAL,\
-                             NETWORKINGCAPITAL,TANGIBLEASSET,RETAINED,INTERESTLIBILITY,NETLIBILITY,EXINTERESTCL,EXINTERESTNCL,FCFF,FCFE,DA,FCFFDRIVE',
-                            f'ReportDate={end_date},Ispandas=1')
+                             NETWORKINGCAPITAL,TANGIBLEASSET,RETAINED,INTERESTLIBILITY,NETLIBILITY,EXINTERESTCL,EXINTERESTNCL,FCFF,FCFE,DA,FCFFDRIVE,\
+                             PERFORMANCEEXPRESSCUTPARENTNI,PERFORMANCEEXPRESSPARENTNI,MBSALESCONS,GPMARGIN,NPMARGIN,EXPENSETOOR,INVTURNRATIO,ARTURNRATIO,ROEAVG,ROEWA,EPSBASIC,EPSDILUTED,\
+                             BPS,BALANCESTATEMENT_25,BALANCESTATEMENT_46,BALANCESTATEMENT_93,BALANCESTATEMENT_103,BALANCESTATEMENT_141,\
+                             BALANCESTATEMENT_140,INCOMESTATEMENT_9,INCOMESTATEMENT_48,INCOMESTATEMENT_60,INCOMESTATEMENT_61,INCOMESTATEMENT_85,INCOMESTATEMENT_127,\
+                             INCOMESTATEMENT_14,CASHFLOWSTATEMENT_39,CASHFLOWSTATEMENT_59,CASHFLOWSTATEMENT_77,CASHFLOWSTATEMENT_82,CASHFLOWSTATEMENT_86',
+                            f'ReportDate={end_date},type=1,Ispandas=1')
         if not isinstance(df_no_param, pd.DataFrame) and df_no_param.ErrorCode != 0:
             print(f'failed to get stock financial factors: {df_no_param.ErrorMsg}')
         else:
             df_list.append(df_no_param)
 
+        # 获取主营收入构成(按产品)
+        df_mb_sales_cons = c.css(stock_id_result.Codes, "MBSALESCONS", f'ReportDate={end_date},type=2,Ispandas=1')
+        if not isinstance(df_mb_sales_cons, pd.DataFrame) and df_mb_sales_cons.ErrorCode != 0:
+            print(f'failed to get stock financial factors: {df_mb_sales_cons.ErrorMsg}')
+        else:
+            df_list.append(df_mb_sales_cons.rename(columns={'MBSALESCONS': 'MBSALESCONS_P'}))
+
         # 获取归属于上市公司股东的扣除非经常性损益后的净利润(调整前)
         df_with_param_1 = c.css(stock_id_result.Codes, "DEDUCTEDINCOME", f'ReportDate={end_date},DataAdjustType=1,Ispandas=1')
         if not isinstance(df_with_param_1, pd.DataFrame) and df_with_param_1.ErrorCode != 0:
@@ -412,6 +511,72 @@ class EmRawDataDownloader(object):
             # 更新到db
             self._upload_raw(df, EmStockFinFac.__table__.name)
 
+    def em_stock_daily_info(self, start_date: str, end_date: str, predict_year: int = 0):
+        try:
+            # 获取区间内所有交易日
+            tradedates = c.tradedates(start_date, end_date, "period=1,order=1,market=CNSESH")
+            if tradedates.ErrorCode != 0:
+                print(f'[em_stock_daily_info] failed to get trade dates: {tradedates.ErrorMsg}')
+                return
+            if len(tradedates.Data) == 0:
+                print(f'[em_stock_daily_info] no trade dates, return immediately (start_date){start_date} (end_date){end_date}')
+                return
+
+            # 获取所有A股股票的ID, 功能函数-板块成分
+            # http://quantapi.eastmoney.com/Cmd/Sector?from=web
+            # 全部A股股票
+            stock_id_result = c.sector('001004', end_date)
+            if stock_id_result.ErrorCode != 0:
+                print(f'failed to get stock id list: {stock_id_result.ErrorMsg}')
+                return
+
+            if predict_year == 0:
+                predict_year = datetime.datetime.strptime(end_date, '%Y%m%d').date().year
+
+            df_list = []
+            for date in tradedates.Data:
+                temp_df = c.css(stock_id_result.Codes,
+                                'TOTALSHARE,HOLDFROZENAMTACCUMRATIO,ASSETMRQ,EQUITYMRQ,PETTMDEDUCTED,PBLYRN,PSTTM,AHOLDER,ESTPEG',
+                                f'EndDate={date},TradeDate={date},PREDICTYEAR={predict_year},Ispandas=1')
+                if not isinstance(temp_df, pd.DataFrame) and temp_df.ErrorCode != 0:
+                    print(f'failed to get stock daily info: {temp_df.ErrorMsg} (date){date}')
+                    return
+                # 日期列修改为end_date
+                temp_df.loc[:, 'DATES'] = date
+                df_list.append(temp_df)
+
+            df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockDailyInfo.__getattribute__(EmStockDailyInfo, x).name)
+            # 更新到db
+            self._upload_raw(df, EmStockDailyInfo.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False
+
+    def em_stock_refinancing(self, start_date: str, end_date: str):
+        try:
+            data = c.ctr("AdditionPlanInfo",
+                         "SECURITYCODE,APPROVENOTICEDATE,PLANNOTICEDDATE,NEWPROGRESS,SUMFINA_T,ATTACHNAME,ADDPURPOSE",
+                        f"DateType=2,StartDate={start_date},EndDate={end_date},Process=0")
+            if data.ErrorCode != 0:
+                print(f'failed to get stock refinancing info: {data.ErrorMsg}')
+                return False
+
+            infos: List[List] = []
+            for v in data.Data.values():
+                # if v[3] in ('董事会批准', '董事会修改', '董事会终止', '股东大会批准', '股东大会修改'):
+                #    continue
+                infos.append(v)
+            df = pd.DataFrame(infos, columns=['SECURITYCODE', 'APPROVENOTICEDATE', 'PLANNOTICEDDATE',
+                                              'NEWPROGRESS', 'SUMFINA_T', 'ATTACHNAME', 'ADDPURPOSE'])
+            self._upload_raw(df.rename(columns=lambda x: EmStockRefinancing.__getattribute__(EmStockRefinancing, x).name), EmStockRefinancing.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False
+
     def em_index_val(self, start_date: str, end_date: str):
         try:
             # 获取区间内所有交易日
@@ -469,30 +634,66 @@ class EmRawDataDownloader(object):
             traceback.print_stack()
             return False
 
+    def em_tradedates(self, start_date, end_date):
+        try:
+            tradedates_result = c.tradedates(start_date, end_date,"period=1,order=1,market=CNSESH")
+            if tradedates_result.ErrorCode != 0:
+                print(f'Failed to get tradedates: {tradedates_result.ErrorMsg}')
+                return False
+
+            tradedates = tradedates_result.Data
+            df = pd.DataFrame(tradedates, columns=['TRADEDATES'])
+            self._upload_raw(df, EmTradeDates.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False
+
+
     def download_all(self, start_date, end_date):
         failed_tasks = []
+        # If em_tradedates fails, there is no trading day between start_date and end_date
+        # Stop and return
+        if not self.em_tradedates(start_date, end_date):
+            failed_tasks.append('em_tradedates')
+            return failed_tasks
+
+        if not self.em_fund_scale(start_date, end_date):
+            failed_tasks.append('em_fund_scale')
+
         if not self.em_fund_nav(start_date, end_date):
             failed_tasks.append('em_fund_nav')
 
-        if not self.em_index_price('H00140.SH', start_date, end_date):
+        if not self.em_index_price(['H00140.SH','H11025.CSI','AU0.SHF'], start_date, end_date):
             failed_tasks.append('em_index_price')
 
         if not self.em_stock_price(start_date, end_date):
             failed_tasks.append('em_stock_price')
+        
+        if not self.em_stock_post_price(start_date, end_date):
+            failed_tasks.append('em_stock_post_price')
 
         if not self.em_index_val(start_date, end_date):
             failed_tasks.append('em_index_val')
         
+        if not self.em_stock_daily_info(start_date, end_date):
+            failed_tasks.append('em_stock_daily_info')
+
         return failed_tasks
 
 
 if __name__ == '__main__':
     em = EmRawDataDownloader()
-    em.em_index_val('2020-04-28', '2020-04-28')
-    # em.em_fund_nav('2020-04-15', '2020-04-27')
+    # em.em_index_val('20200428', '20200428')
+    # em.em_fund_nav('20200415', '20200427')
     # em.em_index_price_history('H00140.SH', '20050101', '20200419')
     # em.em_fund_scale_history()
-    # em.em_stock_fin_fac('2019-06-30')
-    # em.em_stock_info('2020-04-24')
-    # em.em_stock_price('2020-04-28', '2020-04-28')
+    # em.em_stock_fin_fac('20161231')
+    # em.em_stock_info('20200424')
+    # em.em_stock_price('20200428', '20200428')
+    # em.em_stock_daily_info('20200228', '20200228')
     # em.em_stock_price_history('./surfing/data/fund/raw/stock_price')
+    # em.em_stock_refinancing('20170101', '20181231')
+    # em.add_columns_to_fin_fac('20190930')
+    em.em_fund_scale('20200513', '20200513')
