@@ -8,10 +8,17 @@ from ...wrapper.mysql import RawDatabaseConnector
 
 class WebRawDataDownloader(object):
     
+    def __init__(self):
+        self.updated_count = {}
+
     def upload_raw(self, df, table_name):
         print(table_name)
         # print(df)
         df.to_sql(table_name, RawDatabaseConnector().get_engine(), index=False, if_exists='append')
+
+        if table_name not in self.updated_count:
+            self.updated_count[table_name] = 0
+        self.updated_count[table_name] += df.shape[0]
     
     def read_csv(self, csv_path):
         return pd.read_csv(csv_path, index_col = 1)
@@ -47,7 +54,9 @@ class WebRawDataDownloader(object):
             end_date = datetime.datetime.strptime(end_date, '%Y%m%d').strftime('%Y-%m-%d')
             url = ('http://www.chinamoney.com.cn/dqs/rest/cm-u-bk-ccpr/CcprHisExcelNew'
                 f'?startDate={start_date}&endDate={end_date}&currency=USD/CNY,EUR/CNY,100JPY/CNY')
-            r = requests.get(url)
+            headers = {'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/605.1.15 '
+                                      '(KHTML, like Gecko) Version/13.0.5 Safari/605.1.15')}
+            r = requests.get(url, headers=headers)
             filename = os.path.join('/tmp', 'cm.xlsx')
             with open(filename, 'wb') as f:
                 f.write(r.content)
@@ -179,3 +188,6 @@ class WebRawDataDownloader(object):
 
         return failed_tasks
         
+if __name__ == "__main__":
+    web_downloader = WebRawDataDownloader()
+    web_downloader.cm_index_price('20200408', '20200508')
