@@ -4,7 +4,7 @@ import dataclasses
 import datetime
 import time
 from ..wrapper.mysql import RawDatabaseConnector, BasicDatabaseConnector, DerivedDatabaseConnector
-from ..struct import AssetWeight, AssetPrice, FundScoreParam, FundScoreHelper
+from ..struct import AssetWeight, AssetPrice, FundScoreParam, FundScoreHelper, TaaTunerParam
 from .score import FundScoreManager
 from .data_tables import FundDataTables
 from ..view.raw_models import *
@@ -43,10 +43,14 @@ class FundDataManager(DataManager):
             _tm_basic_start = time.time()
             self.dts.fund_info = fetch_table(FundInfo)
             _tm_basic_fetch_info = time.time()
+            no_mmf_index_list = list(set(index_list) - set(['mmf']))
             _filter_fund_info = self.dts.fund_info[
-                    self.dts.fund_info.index_id.isin(index_list)
+                    self.dts.fund_info.index_id.isin(no_mmf_index_list)
                     & ((self.dts.fund_info.structure_type == 0) | (self.dts.fund_info.structure_type == 1)) 
-                    & (self.dts.fund_info.is_etf != 1)
+                    & (self.dts.fund_info.is_etf != 1) 
+                    & (self.dts.fund_info.currency == 'CNY')
+                    & (self.dts.fund_info.is_open == 1)
+                    | (self.dts.fund_info.is_selected_mmf == 1)
                 ]
             _tm_basic_filter_info = time.time()
             self.dts.fund_list = set(_filter_fund_info.fund_id)
@@ -57,7 +61,7 @@ class FundDataManager(DataManager):
             # index
             self.dts.index_info = fetch_table(IndexInfo)
             _tm_basic_fetch_index_info = time.time()
-            _index_query = quant_session.query(IndexPrice.index_id, IndexPrice.datetime, IndexPrice.close).filter(IndexPrice.index_id.in_(index_list))
+            _index_query = quant_session.query(IndexPrice.index_id, IndexPrice.datetime, IndexPrice.close).filter(IndexPrice.index_id.in_(index_list), IndexPrice.datetime <= self.end_date)
             self.dts.index_price = pd.read_sql(_index_query.statement, _index_query.session.bind).pivot_table(index='datetime', columns='index_id', values='close').fillna(method= 'ffill')
             _tm_basic_fetch_index_price = time.time()
             # fund nav
@@ -107,25 +111,34 @@ class FundDataManager(DataManager):
             _tm_deriv_fetch_fund_indicator = time.time()
             self.dts.fund_indicator['index_id'] = self.dts.fund_indicator.fund_id.apply(lambda x: self.dts.fund_index_map[x])
             _tm_deriv_apply_fund_indicator = time.time()
+            # fetch index val data from derived table, and choose different pct value for different index_id
+            _tm_deriv_fetch_index_pct = time.time()
+            pct_index_list = list(TaaTunerParam.POOL.keys())
+            _index_pct_query = derived_session.query(
+                    IndexValuationDevelop
+                ).filter(
+                    IndexValuationDevelop.index_id.in_(pct_index_list),
+                    IndexValuationDevelop.datetime >= self.start_date,
+                    IndexValuationDevelop.datetime <= self.end_date,
+                ).order_by(IndexValuationDevelop.datetime.asc())
+            self._index_pct_df = pd.read_sql(_index_pct_query.statement, _index_pct_query.session.bind).set_index('index_id')
+            # 按早trading date 作为index, fillna 
+            date_list = self.dts.trading_days.datetime.values.tolist()
+            res = []
+            for index_id in pct_index_list:
+                df = self._index_pct_df[self._index_pct_df.index == index_id].set_index('datetime').reindex(date_list).reset_index().fillna(method='ffill')
+                df.loc[:,'index_id'] = index_id
+                res.append(df)
+            _index_pct_df = pd.concat(res, axis=0)
+            self.dts.index_pct = _index_pct_df.pivot_table(index=['datetime','index_id'], values=['pe_pct','pb_pct','ps_pct'])
+            self.dts.index_date_list = self.dts.index_pct.index.remove_unused_levels().levels[0]
+            _tm_deriv_apply_index_pct = time.time()
             if print_time:
                 print(f'\t[time][deriv] fetch fund indicator: {_tm_deriv_fetch_fund_indicator - _tm_deriv_start}')
                 print(f'\t[time][deriv] apply fund indicator: {_tm_deriv_apply_fund_indicator - _tm_deriv_fetch_fund_indicator}')
-        
+                print(f'\t[time][deriv] fetch index val: {_tm_deriv_fetch_index_pct - _tm_deriv_apply_fund_indicator}')
+                print(f'\t[time][deriv] pivot raw index: {_tm_deriv_apply_index_pct - _tm_deriv_fetch_index_pct}')
         _tm_derived = time.time()
-        # raw index val pct, only used in tactic asset allocation
-        with RawDatabaseConnector().managed_session() as quant_session:
-            _tm_raw_start = time.time()
-            #query = quant_session.query(IndexValPct)
-            query = quant_session.query(EMIndexPct) #东财数据新作的raw 指数估值数据 TODO load from derived and gem use ps_pct
-            df = pd.read_sql(query.statement, query.session.bind).pivot_table(index='datetime', columns='index_id', values='pe_pct').fillna(method='ffill')
-            _tm_raw_fetch_index_val = time.time()
-            self.dts.index_pct = df.rename(columns=FundScoreHelper.TACTIC_ASSET_CONVERT)
-            _tm_raw_pivot_index_val = time.time()
-            if print_time:
-                print(f'\t[time][raw] fetch index val: {_tm_raw_fetch_index_val - _tm_raw_start}')
-                print(f'\t[time][raw] pivot raw index: {_tm_raw_pivot_index_val - _tm_raw_fetch_index_val}')
-        _tm_raw = time.time()
-
         self.dts.fix_data()
         _tm_fix = time.time()
 
@@ -133,13 +146,13 @@ class FundDataManager(DataManager):
         self._score_manager.set_dts(self.dts)
         if score_pre_calc:
             self._score_manager.pre_calculate()
+            self._score_manager.remove_a_score_if_c_exsit(self.dts.fund_info)
         _tm_finish = time.time()
         print(self.dts)
         if print_time:
             print(f'[time] basic: {_tm_basic - _tm_start}')
             print(f'[time] deriv: {_tm_derived - _tm_basic}')
-            print(f'[time] raw_d: {_tm_raw - _tm_derived}')
-            print(f'[time] fix_d: {_tm_fix - _tm_raw}')
+            print(f'[time] fix_d: {_tm_fix - _tm_derived}')
             print(f'[time] score: {_tm_finish - _tm_fix}')
             print(f'[time] total: {_tm_finish - _tm_start}')
 
@@ -157,6 +170,11 @@ class FundDataManager(DataManager):
                 res[index_id] = _filtered.iloc[-1]
         return res
 
+    def get_index_pcts_df(self, dt):
+        if dt not in self.dts.index_date_list:
+            return pd.DataFrame()
+        return self.dts.index_pct.loc[dt]
+
     def get_fund_score(self, dt, index_id) -> dict: 
         return self._score_manager.get_fund_score(dt, index_id)
 
@@ -168,43 +186,17 @@ class FundDataManager(DataManager):
         return df.loc[dt].to_dict()
 
     def get_fund_unit_nav(self, dt):
-        df = self.dts.fund_net_nav
+        df = self.dts.fund_unit_nav
         return df.loc[dt].to_dict()
 
-    def get_fund_info(self, fund_id=None):
-        '''
-        fund_id                                                   000001!0
-        wind_id                                                  000001.OF
-        transition                                                       0
-        order_book_id                                               000001
-        desc_name                                                     华夏成长
-        start_date                                              2001-12-18
-        end_date                                                2040-12-31
-        wind_class_1                                                 混合型基金
-        wind_class_2                                               偏股混合型基金
-        manager_id       王亚伟(20011218-20050412)\r\n田擎(20040227-20051029...
-        company_id                                                    华夏基金
-        benchmark                                                        0
-        full_name                                               华夏成长证券投资基金
-        currency                                                       CNY
-        base_fund_id                                                     0
-        is_structured                                                    0
-        is_open                                                          0
-        asset_type                                                    null
-        update_time                                             2020-03-24
-        manage_fee                                                     1.5
-        trustee_fee                                                   0.25
-        purchase_fee                                                   NaN
-        redeem_fee                                                     NaN
-        note                                                          None
-        track_index                                                   none
-        '''
-        if fund_id:
-            df = self.dts.fund_info
-            _filtered = df[df['fund_id']=='000001!0']
-            return None if len(_filtered) == 0 else _filtered.iloc[0]
-        else:
-            return self.dts.fund_info.copy()
+    def get_fund_purchase_fees(self):
+        return self.dts.fund_info.set_index('fund_id').purchase_fee.to_dict()
+
+    def get_fund_redeem_fees(self):
+        return self.dts.fund_info.set_index('fund_id').redeem_fee.to_dict()
+
+    def get_fund_info(self):
+        return self.dts.fund_info
 
     def get_trading_days(self):
         return self.dts.trading_days.copy()
