@@ -1,6 +1,8 @@
 from ..struct import FundScoreParam
 from .data_tables import FundDataTables
 import dataclasses
+import pandas as pd 
+import re
 
 @dataclasses.dataclass
 class ScoreFunc:
@@ -21,11 +23,12 @@ class FundScoreManager:
         self.params = None
         self.dts = None
         self.funcs = {
-            'hs300': ScoreFunc(alpha=0.3, beta=-0.1, fee_rate=-0.1, ir=0.5),
-            'csi500': ScoreFunc(alpha=0.3, beta=-0.1, fee_rate=-0.1, ir=0.5),
-            'gem': ScoreFunc(alpha=0.3, beta=-0.1, fee_rate=-0.1, ir=0.5),
+            'hs300': ScoreFunc(alpha=0.3, beta=-0.6, fee_rate=-0.1),
+            'csi500': ScoreFunc(alpha=0.8, beta=-0.2),
+            'gem': ScoreFunc(alpha=0.2, beta=-0.4, track_err=-0.4),
 
             'national_debt': ScoreFunc(alpha=0.6, fee_rate=-0.2, track_err=-0.2),
+            'mmf':ScoreFunc(alpha=0.6, fee_rate=-0.3, track_err=-0.1),
             'credit_debt': ScoreFunc(alpha=0.2, fee_rate=-0.2, track_err=-0.8),
 
             'sp500rmb': ScoreFunc(fee_rate=-0.2, track_err=-0.8),
@@ -35,6 +38,7 @@ class FundScoreManager:
             'gold': ScoreFunc(alpha=0.2, fee_rate=-0.2, track_err=-0.8),
             'oil': ScoreFunc(alpha=0.2, fee_rate=-0.2, track_err=-0.8),
             'real_state': ScoreFunc(alpha=0.2, fee_rate=-0.2, track_err=-0.8),
+
         }
         self.score_cache = None
         self.score_raw_cache = None
@@ -82,10 +86,41 @@ class FundScoreManager:
                 self.score_cache[dt] = {}
                 self.score_raw_cache[dt] = {}
             cur_d = self.dts.index_fund_indicator_pack.loc[index_id, dt]
-            cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+            if cur_d.shape[0] > 1:
+                cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+            else:
+                cur_d_sta = cur_d
             cur_d_sta['beta'] = cur_d['beta']
             func = self.funcs.get(index_id)
             score_raw = cur_d_sta.apply(lambda x: func.get(x), axis=1)
             score = (score_raw - score_raw.min()) / (score_raw.max() - score_raw.min())
+            if score.shape[0] == 1:
+                score.values[0] = 1
             self.score_cache[dt][index_id] = { fund_id: s for fund_id, s in score.iteritems() }
-            self.score_raw_cache[dt][index_id] = { fund_id: s for fund_id, s in score_raw.iteritems() }
\ No newline at end of file
+            self.score_raw_cache[dt][index_id] = { fund_id: s for fund_id, s in score_raw.iteritems() }
+
+    def filter_score(self, score_dict, fund_id_to_name, fund_name_to_id):
+        # 同一天同资产 同基金A B 在 选B
+        # 同一天同资产 同基金A C 在 选C
+        check_char_list = ['B','C']
+        for check_char in check_char_list:
+            for d, score_d in score_dict.items():
+                for index_id, score_index in score_d.items():
+                    index_fund_name = [fund_id_to_name[_] for _ in list(score_index.keys())]
+                    c_list = [word for word in index_fund_name if check_char in word]
+                    # 如果d日, index_id资产下含有C类基金（包含假C类 如名字含有MSCI）
+                    if len(c_list) > 0:
+                        for c_fund in c_list:
+                            # 替换基金名字里最后一个C 变为A， 这样可以处理假C类基金
+                            a_fund = re.sub(f'{check_char}$', 'A', c_fund)
+                            # 如果A类有评分， 赋值0
+                            if a_fund in index_fund_name:
+                                a_fund_id = fund_name_to_id[a_fund]
+                                score_dict[d][index_id][a_fund_id] = 0
+        return  score_dict
+
+    def remove_a_score_if_c_exsit(self, fund_info:pd.DataFrame):
+        fund_id_to_name = fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
+        fund_name_to_id = fund_info[['fund_id','desc_name']].set_index('desc_name').to_dict()['fund_id']
+        self.score_cache = self.filter_score(self.score_cache, fund_id_to_name, fund_name_to_id)
+        self.score_raw_cache = self.filter_score(self.score_raw_cache, fund_id_to_name, fund_name_to_id)
\ No newline at end of file
