@@ -19,6 +19,7 @@ class AssetDict:
     oil: float=0.0
     n225rmb: float=0.0
     cash: float=0.0
+    mmf: float=0.0
 
     def __repr__(self):
         s = f'<{self.__class__.__name__}'
@@ -105,7 +106,7 @@ class AssetPosition(AssetDict):
     def update(self, trd: AssetTrade):
         p = trd.trade_price if trd.trade_price else trd.mark_price
         v = trd.volume if trd.volume else trd.amount / p
-        assert trd.is_buy or (self.__dict__[trd.index_id] - v > -1e-8), f'asset trade failed, asset trade sell volume {v} more than position {self.__dict__[trd.index_id]}'
+        assert trd.is_buy or (self.__dict__[trd.index_id] - v > -1e-8), f'asset trade failed, (sell){v} (cur){self.__dict__[trd.index_id]} (trd){trd} (pos){self}'
         amt = trd.amount if trd.amount else v * p
         self.__dict__[trd.index_id] += v if trd.is_buy else -v
         self.cash += (-amt if trd.is_buy else amt) - trd.commission
@@ -148,6 +149,8 @@ class AssetValue(AssetDict):
 class FundTrade(AssetTrade):
 
     fund_id: str=None
+    fund_unit_nav: float=None
+    fund_unit_volume: float=None
 
 @dataclasses.dataclass
 class FundPosItem:
@@ -162,14 +165,15 @@ class FundPosItem:
         assert self.fund_id == trade.fund_id, f'fund trade failed, self.fund_id != trade.fund_id: self.fund_id {self.fund_id}  trade.fund_id {trade.fund_id}'
         p = trade.trade_price if trade.trade_price else trade.mark_price
         v = trade.volume if trade.volume else trade.amount / p
+        a = trade.amount if trade.amount else v * p
         assert trade.is_buy or (self.volume - v > -1e-8), f'fund trade failed, trade.volume {v} more than self.volume {self.volume}'
         vol_diff = v if trade.is_buy else -v
-        amt_diff = trade.amount if trade.is_buy else -trade.amount
+        amt_diff = a if trade.is_buy else -a
         volume = self.volume + vol_diff
         if trade.is_buy:
             amt = self.volume * self.avg_cost + amt_diff# 买入时平均成本摊销
             self.volume = volume
-            self.avg_cost = amt / volume 
+            self.avg_cost = amt / volume
         else:
             self.volume = volume
             if self.volume < 1e-8:# 清空仓位 平均成本为0
@@ -184,18 +188,47 @@ class FundPosItem:
     def __repr__(self):
         return f'<{self.fund_id} (v){self.volume} (c){self.avg_cost}>'
 
+@dataclasses.dataclass
+class AssetTradeRateItem(AssetDict):
+    def __post_init__(self):
+        for k, v in self.__dict__.items():
+            self.__dict__[k] = None if (v == 0 and k != 'mmf') else v
+
 @dataclasses.dataclass
 class AssetTradeParam:
 
     MinCountAmtDiff: float=0.001 # 少于总资产千分之一的资产Amount差异，不计在内
     MinActionAmtDiff: float=0.03 # 如果没有资产的偏离达到这个比例，那就不调仓
+    EnableCommission: bool=False # 是否开启交易手续费计算
+    PurchaseDiscount: float=1.0  # 大类资产的买入折扣
+    RedeemDiscount: float=1.0    # 大类资产的卖出折扣
+    AssetPurchaseRate: dict=None 
+    AssetRedeemRate: dict=None
 
+    def __post_init__(self):
+        self.AssetPurchaseRate = self.AssetPurchaseRate or AssetTradeRateItem(csi500=0.011055, 
+                                                                                gem=0.010488, 
+                                                                                gold=0.001892, 
+                                                                                hs300=0.011987, 
+                                                                                national_debt=0.005438, 
+                                                                                sp500rmb=0.008135,
+                                                                                mmf=0).__dict__ #大类资产平均买入费率
+        self.AssetRedeemRate = self.AssetRedeemRate or AssetTradeRateItem(csi500=0.002973, 
+                                                                            gem=0.003407,
+                                                                            gold=0.000755,
+                                                                            hs300=0.004727,
+                                                                            national_debt=0.000287,
+                                                                            sp500rmb=0.003240,
+                                                                            mmf=0).__dict__ #大类资产平均卖出费率
 @dataclasses.dataclass
 class FundTradeParam:
 
     MaxFundNumUnderAsset: float=3 # 同一个大类资产下基金最多数
-    MinCountAmtDiff: float=0.003 # 少于总资产千分之三的资产Amount差异，不做基金调仓处理
+    MinCountAmtDiff: float=0.0003 # 少于总资产万分之三的资产Amount差异，不做基金调仓处理
     MinActionAmtDiff: float=0.03 # 如果没有资产的基金市值偏离达到这个比例，那就不调仓
+    EnableCommission: bool=False # 是否开启交易手续费计算
+    PurchaseDiscount: float=1.0  # 基金的买入折扣
+    RedeemDiscount: float=1.0    # 基金的卖出折扣
 
 @dataclasses.dataclass
 class TAAParam:
@@ -206,6 +239,8 @@ class TAAParam:
     LowPlus:      float=0.05    # 估值偏低是应该调高的占比
     HighStop:     float=0.5     # 进入估值偏高区后，什么时候算作退出
     LowStop:      float=0.5     # 进入估值偏低区后，什么时候算作退出
+    TuneCash:     bool=False    # 默认taa调仓时，不调整cash
+    ToMmf:        bool=False    # 默认taa加减仓不改变货基权重
 
 @dataclasses.dataclass
 class FundScoreParam:
@@ -218,6 +253,48 @@ class FundScoreParam:
         return f'<fund_score_param tag_type={self.tag_type} score_method={self.score_method} is_full={self.is_full}'
 
 
+@dataclasses.dataclass
+class FAParam:
+
+    MaxFundNumUnderAsset: float=3 # 同一个大类资产下基金最多数
+
+@dataclasses.dataclass
+class FundWeightItem:
+
+    fund_id: str                # 基金ID
+    index_id: str               # 基金所属的大类资产
+    asset_wgt: float            # 资产的wgt
+    fund_wgt_in_asset: float    # 基金在所属资产中的wgt
+    fund_wgt: float=None        # 基金在整体组合中的wgt
+
+    def __post_init__(self):
+        self.fund_wgt = self.fund_wgt or self.asset_wgt * self.fund_wgt_in_asset
+
+    def __repr__(self):
+        return f'<{self.fund_id} (wgt){self.fund_wgt} (index){self.index_id} (iwgt){self.asset_wgt} (fwgt){self.fund_wgt_in_asset}>'
+
+class FundWeight:
+
+    def __init__(self):
+        self.funds = {} # fund_id -> FundWeightItem
+
+    def add(self, fund_wgt_item: FundWeightItem):
+        self.funds[fund_wgt_item.fund_id] = fund_wgt_item
+    
+    def get_wgt(self, fund_id: str):
+        fund_wgt_item = self.funds.get(fund_id)
+        return fund_wgt_item.fund_wgt if fund_wgt_item else 0
+    
+    def get(self, fund_id: str):
+        return self.funds.get(fund_id)
+
+    def __repr__(self):
+        s = f'<FW'
+        for fund_id, wgt_item in self.funds.items():
+            s += f' {wgt_item}'
+        s += '>'
+        return s
+
 class FundPosition:
 
     def __init__(self, cash):
@@ -225,6 +302,19 @@ class FundPosition:
         self.index_fund_list = {} # index_id -> [fund_id]
         self.cash = cash
 
+    def init_by_item_dict(self, cash, funds, index_fund_list):
+        self.cash = cash
+        self.funds = funds
+        self.index_fund_list = index_fund_list
+        return self
+
+    def convert_to_item_dict(self):
+        return {
+            'cash': self.cash,
+            'funds': {f: v.__dict__ for f, v in self.funds.items()},
+            'index_fund_list': self.index_fund_list,
+        }
+
     def calc_mv_n_w(self, 
                     fund_navs: dict, # fund_id -> nav(adjusted net value)
                     index_id=None): # filter: 只计算某个index_id
@@ -275,13 +365,15 @@ class FundPosition:
         return pos_item
 
     def update(self, trade: FundTrade):
+        assert isinstance(trade, FundTrade), 'only FundTrade is accepted here'
         pos_item = self.funds.get(trade.fund_id)
         if pos_item is None:
             assert trade.is_buy, "fund volume cannot be negative"
             pos_item = self.add(trade.fund_id, trade.index_id, 0, 0, trade.trade_date or trade.submit_date)
             
         if pos_item.update(trade):
-            amt = trade.amount
+            p = trade.trade_date if trade.trade_price else trade.mark_price
+            amt = trade.amount if trade.amount else trade.volume * p
             self.cash += (-amt if trade.is_buy else amt) - trade.commission
             assert self.cash >= -1e-8, f"fund backtest cash should not be negative! (cash){self.cash} (trade){trade}"
             return True
@@ -349,6 +441,23 @@ class FundScoreHelper:
     def parse(tag_name):
         return FundScoreHelper.ASSET_DICT.get(tag_name)
 
+class TaaTunerParam:
+    index_id:            str
+    IsTaaLowOnly:        bool=False
+    IsTaaUpOnly:         bool=False
+    param_list: list=[] # 调参范围列表
+    POOL = {
+        'gem'   : 'ps_pct',
+        'hs300' : 'pe_pct',
+        'csi500': 'pe_pct',
+        'sp500rmb': 'pe_pct', 
+    }
+    PCT_PAIR = {
+        'gem':'ps_ttm',
+        'hs300':'pe_ttm',
+        'csi500':'pe_pct',
+        'sp500rmb':'pe_pct',
+    }
 
 if __name__ == '__main__':
     a = AssetWeight(hs300=0.1, csi500=0.1, cash=0.8)
