@@ -1,8 +1,15 @@
-from sqlalchemy import CHAR, Column, Integer, Text, Index
+from sqlalchemy import CHAR, Column, Integer, Text, Index, BOOLEAN, text
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT
+from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME
 
-Base = declarative_base()
+
+class Base():
+    _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
+
+
+# make this column at the end of every derived table
+Base._update_time._creation_order = 9999
+Base = declarative_base(cls=Base)
 
 
 class IndexInfo(Base):
@@ -34,7 +41,6 @@ class FundSize(Base):
 
     fund_id = Column(CHAR(10), primary_key=True) # 基金id
     latest_size = Column(DOUBLE(asdecimal=False)) # 最新规模
-    update_time = Column(DATE) # 更新日期
 
 
 class FundInfo(Base):
@@ -58,11 +64,12 @@ class FundInfo(Base):
     currency = Column(CHAR(20)) # 币种
     base_fund_id = Column(CHAR(20)) # 分级基金基础基金代号
     is_structured = Column(TINYINT(1)) # 是否为分级基金
-    is_open = Column(TINYINT(1)) # 是否为开放式基金
+    is_open = Column(TINYINT(1)) # 是否为日常开放申赎的基金 1是 0否， 排除掉封闭基金和定期开放基金
+    is_regular_open_ended = Column(TINYINT(1)) # 是否为定期开放式基金 1是 0否
+    is_closed_ended= Column(TINYINT(1)) # 是否为封闭基金 1是 0 否
     structure_type = Column(TINYINT(1))  # 是否为分级子基金  0/nan非   1母   2分级A  3 分级B 4 其他
-    is_etf = Column(TINYINT(1)) #是否是etf  0 非  1 etf
-    asset_type = Column(CHAR(32)) #资产类别
-    update_time = Column(DATE) # 更新时间
+    is_etf = Column(TINYINT(1)) # 是否是etf  0 非  1 etf
+    asset_type = Column(CHAR(32)) # 资产类别
     tt_purchase_fee = Column(DOUBLE(asdecimal=False)) # 天天基金优惠申购费
     manage_fee = Column(DOUBLE(asdecimal=False)) # 管理费
     trustee_fee = Column(DOUBLE(asdecimal=False)) # 托管费
@@ -73,8 +80,11 @@ class FundInfo(Base):
     benchmark_1 = Column(CHAR(255)) # 业绩基准标的指数简称第一名
     benchmark_2 = Column(CHAR(255)) # 业绩基准标的指数简称第二名
     index_id = Column(CHAR(20)) # 基于第一业绩标准所标注的指数ID
-
-
+    is_c =  Column("is_c", BOOLEAN, nullable=False)  #是否是etf  0 非  1 c  
+    is_a = Column("is_a", BOOLEAN, nullable=False)  #是否是etf  0 非  1 a   
+    ac_filter = Column("ac_filter", BOOLEAN, nullable=False) #是否ac均有要排除的a 0 排除  1 保留  
+    is_selected_mmf = Column(TINYINT(1)) # 是否基金规模最大的前50只货币基金(10年前有两只存在)  0 非  1 是
+    
 class TradingDayList(Base):
     '''交易日列表'''
 
@@ -160,11 +170,11 @@ class FundRet(Base):
     m3_ret = Column(DOUBLE(asdecimal=False)) # 近一季度收益率
     m6_ret = Column(DOUBLE(asdecimal=False)) # 近半年收益率
     y1_ret = Column(DOUBLE(asdecimal=False)) # 近一年收益率
+    to_date_ret = Column(DOUBLE(asdecimal=False)) # 成立至今收益率
     mdd = Column(DOUBLE(asdecimal=False)) # 最大回撤（成立以来）
     annual_ret = Column(DOUBLE(asdecimal=False)) # 年化收益（成立以来）
     avg_size = Column(DOUBLE(asdecimal=False)) # 平均规模（成立以来）
     sharpe_ratio = Column(DOUBLE(asdecimal=False)) # 夏普率（成立以来）
-    info_ratio = Column(DOUBLE(asdecimal=False)) # 夏普率（成立以来）
 
 
 class FundRatingLatest(Base):
