@@ -1,8 +1,15 @@
-from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN
+from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.dialects.mysql import DOUBLE, DATE
+from sqlalchemy.dialects.mysql import DOUBLE, DATE, DATETIME
 
-Base = declarative_base()
+
+class Base():
+    _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
+
+
+# make this column at the end of every derived table
+Base._update_time._creation_order = 9999
+Base = declarative_base(cls=Base)
 
 
 class IndexVolatility(Base):
@@ -163,10 +170,39 @@ class FundScore(Base):
     tag_type = Column(Integer) # 1 '大类资产评分'
     tag_method = Column(CHAR(40)) # 打标签版本号
     is_full = Column(BOOLEAN) # 1 true
-    update_time = Column(DATE) # 更新时间
     score_method = Column(Integer)
     desc_name = Column(CHAR(64))
 
     __table_args__ = (
         Index('idx_fund_score_datetime', 'datetime'),
     )
+
+
+class AssetAllocationInfo(Base):
+    '''十档大类资产组合'''
+
+    __tablename__ = 'asset_allocation_info'
+    allocation_id = Column(Integer, primary_key=True) # 组合id
+    version = Column(Integer, primary_key=True) # 版本号
+
+    hs300 =  Column(DOUBLE(asdecimal=False)) # 沪深300权重
+    csi500 =  Column(DOUBLE(asdecimal=False)) # 中证500权重
+    gem =  Column(DOUBLE(asdecimal=False)) # 创业板权重
+    sp500rmb =  Column(DOUBLE(asdecimal=False)) # 标普500人民币权重
+    national_debt = Column(DOUBLE(asdecimal=False)) # 国债权重
+    gold =  Column(DOUBLE(asdecimal=False)) # 黄金权重
+    credit_debt =  Column(DOUBLE(asdecimal=False)) # 信用债权重
+    dax30rmb = Column(DOUBLE(asdecimal=False)) # 德国dax30权重
+    real_state = Column(DOUBLE(asdecimal=False)) # 房地产权重 拼错了
+    oil = Column(DOUBLE(asdecimal=False)) # 原油权重
+    n225rmb = Column(DOUBLE(asdecimal=False)) # 日经225权重
+    cash = Column(DOUBLE(asdecimal=False)) # 现金权重
+    mdd = Column(DOUBLE(asdecimal=False)) # 最大回撤
+    annual_ret = Column(DOUBLE(asdecimal=False)) # 年化收益
+    sharpe = Column(DOUBLE(asdecimal=False)) # 夏普率
+    recent_y5_ret = Column(DOUBLE(asdecimal=False)) # 最近五年收益
+    annual_vol = Column(DOUBLE(asdecimal=False)) # 年化波动率
+    mdd_d1 = Column(DATE) # 最大回撤开始时间
+    mdd_d2 = Column(DATE) # 最大回撤结束时间
+    start_date = Column(DATE) # 回测开始时间
+    end_date = Column(DATE) # 回测结束时间
