@@ -3,7 +3,14 @@ from sqlalchemy import CHAR, Column, Integer, TEXT, BOOLEAN, text, Index
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME
 
-Base = declarative_base()
+
+class Base():
+    _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
+
+
+# make this column at the end of every derived table
+Base._update_time._creation_order = 9999
+Base = declarative_base(cls=Base)
 
 
 class WindFundInfo(Base):
@@ -41,9 +48,9 @@ class RqStockPrice(Base):
     low = Column(DOUBLE(asdecimal=False)) # 最低价
     limit_up = Column(DOUBLE(asdecimal=False)) # 涨停价
     limit_down = Column(DOUBLE(asdecimal=False)) # 跌停价
-    total_turnover = Column(DOUBLE(asdecimal=False)) # 交易额
-    volume = Column(DOUBLE(asdecimal=False)) # 交易量  不复权和后复权的交易量不同
-    num_trades = Column(DOUBLE(asdecimal=False)) # 交易笔数
+    total_turnover = Column(DOUBLE(asdecimal=False)) # 成交额
+    volume = Column(DOUBLE(asdecimal=False)) # 成交量  不复权和后复权的交易量不同
+    num_trades = Column(DOUBLE(asdecimal=False)) # 成交笔数
 
     __table_args__ = (
         Index('idx_rq_stock_price_datetime', 'datetime'),
@@ -64,9 +71,9 @@ class RqStockPostPrice(Base):
     low = Column(DOUBLE(asdecimal=False)) # 最低价
     limit_up = Column(DOUBLE(asdecimal=False)) # 涨停价
     limit_down = Column(DOUBLE(asdecimal=False)) # 跌停价
-    total_turnover = Column(DOUBLE(asdecimal=False)) # 交易额
-    volume = Column(DOUBLE(asdecimal=False)) # 交易量  不复权和后复权的交易量不同
-    num_trades = Column(DOUBLE(asdecimal=False)) # 交易笔数
+    total_turnover = Column(DOUBLE(asdecimal=False)) # 成交额
+    volume = Column(DOUBLE(asdecimal=False)) # 成交量  不复权和后复权的交易量不同
+    num_trades = Column(DOUBLE(asdecimal=False)) # 成交笔数
 
     __table_args__ = (
         Index('idx_rq_stock_post_price_datetime', 'datetime'),
@@ -82,12 +89,12 @@ class RqFundNav(Base):
     datetime = Column(DATE, primary_key=True) # 日期
 
     unit_net_value = Column(DOUBLE(asdecimal=False)) # 单位净值
-    acc_net_value = Column(DOUBLE(asdecimal=False)) # 累积单位净值
+    acc_net_value = Column(DOUBLE(asdecimal=False)) # 累计单位净值
     adjusted_net_value = Column(DOUBLE(asdecimal=False)) # 复权净值
     change_rate = Column(DOUBLE(asdecimal=False)) # 涨跌幅
     daily_profit = Column(DOUBLE(asdecimal=False)) # 每万元收益（日结型货币基金专用）
     weekly_yield = Column(DOUBLE(asdecimal=False)) # 7日年化收益率（日结型货币基金专用）
-    subscribe_status = Column(CHAR(10)) # 订阅状态，开放 - Open, 暂停 - Suspended, 限制大额申赎 - Limited, 封闭期 - Close
+    subscribe_status = Column(CHAR(10)) # 申购状态，开放 - Open, 暂停 - Suspended, 限制大额申赎 - Limited, 封闭期 - Close
     redeem_status = Column(CHAR(10)) # 赎回状态，开放 - Open, 暂停 - Suspended, 限制大额申赎 - Limited, 封闭期 - Close
 
     __table_args__ = (
@@ -115,7 +122,7 @@ class RqStockValuation(Base):
     pcf_ratio_total_lyr = Column(DOUBLE(asdecimal=False)) # 市现率_总现金流lyr
     pcf_ratio_total_ttm = Column(DOUBLE(asdecimal=False)) # 市现率_总现金流ttm
     pcf_ratio_lyr = Column(DOUBLE(asdecimal=False)) # 市现率_经营lyr
-    pcf_ratio_ttm = Column(DOUBLE(asdecimal=False)) # 市现率_经营 ttm
+    pcf_ratio_ttm = Column(DOUBLE(asdecimal=False)) # 市现率_经营ttm
     cfp_ratio_lyr = Column(DOUBLE(asdecimal=False)) # 现金收益率lyr
     cfp_ratio_ttm = Column(DOUBLE(asdecimal=False)) # 现金收益率ttm
     pb_ratio_lyr = Column(DOUBLE(asdecimal=False)) # 市净率lyr
@@ -139,12 +146,12 @@ class RqStockValuation(Base):
     ev_lyr = Column(DOUBLE(asdecimal=False)) # 企业价值lyr
     ev_ttm = Column(DOUBLE(asdecimal=False)) # 企业价值ttm
     ev_lf = Column(DOUBLE(asdecimal=False)) # 企业价值lf
-    ev_no_cash_lyr = Column(DOUBLE(asdecimal=False)) #企业价值(不含货币资金)lyr
+    ev_no_cash_lyr = Column(DOUBLE(asdecimal=False)) # 企业价值(不含货币资金)lyr
     ev_no_cash_ttm = Column(DOUBLE(asdecimal=False)) # 企业价值(不含货币资金)ttm
     ev_no_cash_lf = Column(DOUBLE(asdecimal=False)) # 企业价值(不含货币资金)lf
     ev_to_ebitda_lyr = Column(DOUBLE(asdecimal=False)) # 企业倍数lyr
     ev_to_ebitda_ttm = Column(DOUBLE(asdecimal=False)) # 企业倍数ttm
-    ev_no_cash_to_ebit_lyr = Column(DOUBLE(asdecimal=False)) #企业倍数(不含货币资金)lyr
+    ev_no_cash_to_ebit_lyr = Column(DOUBLE(asdecimal=False)) # 企业倍数(不含货币资金)lyr
     ev_no_cash_to_ebit_ttm = Column(DOUBLE(asdecimal=False)) # 企业倍数(不含货币资金)ttm
 
     __table_args__ = (
@@ -187,8 +194,8 @@ class RqIndexPrice(Base):
     order_book_id = Column(CHAR(20), primary_key=True) # 米筐代码
     datetime = Column(DATE, primary_key=True) # 日期
 
-    high = Column(DOUBLE(asdecimal=False)) #最高价
-    open = Column(DOUBLE(asdecimal=False)) #开盘价
+    high = Column(DOUBLE(asdecimal=False)) # 最高价
+    open = Column(DOUBLE(asdecimal=False)) # 开盘价
     total_turnover = Column(DOUBLE(asdecimal=False)) # 交易额
     low = Column(DOUBLE(asdecimal=False)) # 最低价
     close = Column(DOUBLE(asdecimal=False)) # 收盘价
@@ -213,7 +220,7 @@ class EmIndexPrice(Base):
     '''Choice指数数据'''
 
     __tablename__ = 'em_index_price'
-    em_id = Column(CHAR(20), primary_key=True) # 东财代码
+    em_id = Column(CHAR(20), primary_key=True) # Choice代码
     datetime = Column(DATE, primary_key=True) # 日期
     close = Column(DOUBLE(asdecimal=False)) # 收盘价
 
@@ -224,7 +231,7 @@ class FundFee(Base):
     __tablename__ = 'fund_fee'
     id = Column(Integer, primary_key=True)
 
-    desc_name = Column(CHAR(32)) # 名称
+    desc_name = Column(CHAR(32)) # 基金名称
     manage_fee = Column(DOUBLE(asdecimal=False)) # 管理费
     trustee_fee = Column(DOUBLE(asdecimal=False)) # 托管费
     purchase_fee = Column(DOUBLE(asdecimal=False)) # 申购费
@@ -306,7 +313,7 @@ class RqIndexWeight(Base):
 
     __tablename__ = 'rq_index_weight'
 
-    index_id = Column(CHAR(20), primary_key=True) # 股票
+    index_id = Column(CHAR(20), primary_key=True) # 指数ID
     datetime = Column(DATE, primary_key=True) # 日期
 
     weight_list = Column(TEXT) # 权重列表 json 格式str 两者顺位对应
@@ -333,7 +340,7 @@ class RqStockFinFac(Base):
     basic_earnings_per_share_lyr_0 = Column(DOUBLE(asdecimal=False))  # 最近一期年报基本每股收益LYR
     basic_earnings_per_share_lyr_1 = Column(DOUBLE(asdecimal=False))  # 最近一期上一期年报基本每股收益LYR
     basic_earnings_per_share_lyr_2 = Column(DOUBLE(asdecimal=False))  # 最近一期上上期年报基本每股收益LYR
-    equity_parent_company_mrq_0 = Column(DOUBLE(asdecimal=False))  #最近一期单季度归母公司所有者权益合计
+    equity_parent_company_mrq_0 = Column(DOUBLE(asdecimal=False))  # 最近一期单季度归母公司所有者权益合计
 
 
 class RqFundSize(Base):
@@ -343,7 +350,6 @@ class RqFundSize(Base):
 
     order_book_id = Column(CHAR(10), primary_key=True) # 米筐基金id
     latest_size = Column(DOUBLE(asdecimal=False)) # 最新规模
-    update_time = Column(DATE) # 更新日期
 
 
 class StockInfo(Base):
@@ -384,7 +390,7 @@ class EmFundNav(Base):
     DATES = Column(DATE, primary_key=True) # 日期
 
     ORIGINALUNIT = Column(DOUBLE(asdecimal=False)) # 单位净值
-    ORIGINALNAVACCUM = Column(DOUBLE(asdecimal=False)) # 累积单位净值
+    ORIGINALNAVACCUM = Column(DOUBLE(asdecimal=False)) # 累计单位净值
     ADJUSTEDNAV = Column(DOUBLE(asdecimal=False)) # 复权净值
     UNITYIELD10K = Column(DOUBLE(asdecimal=False)) # 每万元收益（日结型货币基金专用）
     YIELDOF7DAYS = Column(DOUBLE(asdecimal=False)) # 7日年化收益率（日结型货币基金专用）
@@ -437,7 +443,42 @@ class EmStockPrice(Base):
     TNUM = Column('t_num', DOUBLE(asdecimal=False), server_default=text('0')) # 成交笔数
     BUYVOL = Column('buy_vol', DOUBLE(asdecimal=False), server_default=text('0')) # 内盘成交量
     SELLVOL = Column('sell_vol', DOUBLE(asdecimal=False), server_default=text('0')) # 外盘成交量
-    UPDATE_TIME = Column("update_time", DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP')) # 更新时间
+
+
+class EmStockPostPrice(Base):
+    '''Choice股票后复权数据表'''
+
+    __tablename__ = 'em_stock_post_price'
+
+    CODES = Column('stock_id', CHAR(10), primary_key=True) # EM股票ID
+    DATES = Column('datetime', DATE, primary_key=True) # 日期
+    OPEN = Column('open', DOUBLE(asdecimal=False), nullable=False) # 开盘价
+    CLOSE = Column('close', DOUBLE(asdecimal=False), nullable=False) # 收盘价
+    HIGH = Column('high', DOUBLE(asdecimal=False), nullable=False) # 最高价
+    LOW = Column('low', DOUBLE(asdecimal=False), nullable=False) # 最低价
+    PRECLOSE = Column('pre_close', DOUBLE(asdecimal=False), nullable=False) # 前收盘价
+    AVERAGE = Column('average', DOUBLE(asdecimal=False), server_default=text("0")) # 均价
+    AMOUNT = Column('amount', DOUBLE(asdecimal=False), server_default=text('0')) # 成交额
+    VOLUME = Column('volume', DOUBLE(asdecimal=False), server_default=text('0')) # 成交量
+    TRADESTATUS = Column('trade_status', CHAR(20), nullable=False) # 交易状态
+    TAFACTOR = Column('tafactor', DOUBLE(asdecimal=False), server_default=text("0")) # 复权因子（后）
+    
+class EmStockDailyInfo(Base):
+    '''Choice股票每日更新'''
+
+    __tablename__ = 'em_stock_daily_info'
+
+    CODES = Column("stock_id", CHAR(10), primary_key=True) # EM股票ID
+    DATES = Column('datetime', DATE, primary_key=True) # 日期
+    TOTALSHARE = Column("total_share", DOUBLE(asdecimal=False), nullable=False) # 总股本
+    HOLDFROZENAMTACCUMRATIO = Column('hold_frozen_amt_accum_ratio', DOUBLE(asdecimal=False))  # 控股股东累计质押数量占持股比例
+    ASSETMRQ = Column('asset_mrq', DOUBLE(asdecimal=False))  # 资产总计(MRQ)
+    EQUITYMRQ = Column('equity_mrq', DOUBLE(asdecimal=False))  # 归属母公司股东的权益(MRQ)(净资产)
+    PETTMDEDUCTED = Column('pe_ttm_deducted', DOUBLE(asdecimal=False))  # 市盈率TTM(扣除非经常性损益)
+    PBLYRN = Column('pb_lyr_n', DOUBLE(asdecimal=False))  # 市净率(PB，LYR)(按公告日)
+    PSTTM = Column('ps_ttm', DOUBLE(asdecimal=False))  # 市销率(PS，TTM)
+    AHOLDER = Column('a_holder', TEXT)  # 实际控制人
+    ESTPEG = Column('est_peg', DOUBLE(asdecimal=False)) # 预测PEG(最近一年)
 
 
 class EmStockInfo(Base):
@@ -453,7 +494,7 @@ class EmStockInfo(Base):
     LISTDATE = Column("list_date", DATE, nullable=False) # 首发上市日期
     FINPURCHORNOT = Column("fin_purch_ornot", BOOLEAN, nullable=False) # 是否属于融资标的
     FINSELLORNOT = Column("fin_sell_ornot", BOOLEAN, nullable=False) # 是否属于融券标的
-    STOHSTOCKCONNECTEDORNOT = Column("stoh_stock_connected_ornot", BOOLEAN, nullable=False) #是否属于沪股通标的
+    STOHSTOCKCONNECTEDORNOT = Column("stoh_stock_connected_ornot", BOOLEAN, nullable=False) # 是否属于沪股通标的
     SHENGUTONGTARGET = Column("shengutong_target", BOOLEAN, nullable=False) # 是否属于深股通标的
     BLEMINDCODE = Column("bl_em_ind_code", CHAR(40), nullable=False) # 所属东财行业指数代码
     BLSWSINDCODE = Column("bl_sws_ind_code", CHAR(40), nullable=False) # 所属申万行业指数代码
@@ -464,7 +505,6 @@ class EmStockInfo(Base):
     CSRCCODENEW = Column("csrc_code_new", CHAR(16), nullable=False) # 所属证监会行业(新)代码
     CSINDCODE2016 = Column("cs_ind_code_2016", CHAR(32), nullable=False) # 所属中证行业(2016)代码
     GICSCODE = Column("gics_code", CHAR(32), nullable=False) # 所属GICS行业代码
-    UPDATE_TIME = Column("update_time", DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP')) # 更新时间
 
 
 class EmStockFinFac(Base):
@@ -526,8 +566,53 @@ class EmStockFinFac(Base):
     CFITTMR = Column("cfi_ttmr", DOUBLE(asdecimal=False)) # 投资活动现金净流量TTM(报告期)
     CFFTTMR = Column("cff_ttmr", DOUBLE(asdecimal=False)) # 筹资活动现金净流量TTM(报告期)
     CFTTMR = Column("cf_ttmr", DOUBLE(asdecimal=False)) # 现金净流量TTM(报告期)
-    CAPEXR = Column("cap_exr", DOUBLE(asdecimal=False)) # 资本支出TTM（报告期）
-    UPDATE_TIME = Column("update_time", DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP')) # 更新时间
+    CAPEXR = Column("cap_exr", DOUBLE(asdecimal=False)) # 资本支出TTM(报告期)
+    PERFORMANCEEXPRESSPARENTNI = Column('performance_express_parent_ni', DOUBLE(asdecimal=False)) # 业绩快报.归属母公司股东的净利润
+    MBSALESCONS = Column('mb_sales_cons', TEXT) # 主营收入构成(按行业)
+    MBSALESCONS_P = Column('mb_sales_cons_p', TEXT) # 主营收入构成(按产品)
+    GPMARGIN = Column('gp_margin', DOUBLE(asdecimal=False)) # 销售毛利率
+    NPMARGIN = Column('np_margin', DOUBLE(asdecimal=False)) # 销售净利率(营业收入/净利润)
+    EXPENSETOOR = Column('expense_toor', DOUBLE(asdecimal=False)) # 销售期间费用率
+    INVTURNRATIO = Column('inv_turn_ratio', DOUBLE(asdecimal=False)) # 存货周转率
+    ARTURNRATIO = Column('ar_turn_ratio', DOUBLE(asdecimal=False)) # 应收账款周转率(含应收票据)
+    ROEAVG = Column('roe_avg', DOUBLE(asdecimal=False)) # 净资产收益率ROE(平均)
+    ROEWA = Column('roe_wa', DOUBLE(asdecimal=False)) # 净资产收益率ROE(加权)
+    EPSBASIC = Column('eps_basic', DOUBLE(asdecimal=False)) # 每股收益EPS(基本)
+    EPSDILUTED = Column('eps_diluted', DOUBLE(asdecimal=False)) # 每股收益EPS(稀释)
+    BPS = Column('bps', DOUBLE(asdecimal=False)) # 每股净资产
+    BALANCESTATEMENT_25 = Column('balance_statement_25', DOUBLE(asdecimal=False)) # 流动资产合计
+    BALANCESTATEMENT_46 = Column('balance_statement_46', DOUBLE(asdecimal=False)) # 非流动资产合计
+    BALANCESTATEMENT_93 = Column('balance_statement_93', DOUBLE(asdecimal=False)) # 流动负债合计
+    BALANCESTATEMENT_103 = Column('balance_statement_103', DOUBLE(asdecimal=False)) # 非流动负债合计
+    BALANCESTATEMENT_141 = Column('balance_statement_141', DOUBLE(asdecimal=False)) # 股东权益合计
+    BALANCESTATEMENT_140 = Column('balance_statement_140', DOUBLE(asdecimal=False)) # 归属于母公司股东权益合计
+    INCOMESTATEMENT_9 = Column('income_statement_9', DOUBLE(asdecimal=False)) # 营业收入
+    INCOMESTATEMENT_48 = Column('income_statement_48', DOUBLE(asdecimal=False)) # 营业利润
+    INCOMESTATEMENT_60 = Column('income_statement_60', DOUBLE(asdecimal=False)) # 净利润
+    INCOMESTATEMENT_61 = Column('income_statement_61', DOUBLE(asdecimal=False)) # 归属于母公司股东的净利润
+    INCOMESTATEMENT_85 = Column('income_statement_85', DOUBLE(asdecimal=False)) # 其他业务收入
+    INCOMESTATEMENT_127 = Column('income_statement_127', DOUBLE(asdecimal=False)) # 利息费用
+    INCOMESTATEMENT_14 = Column('income_statement_14', DOUBLE(asdecimal=False)) # 财务费用
+    CASHFLOWSTATEMENT_39 = Column('cashflow_statement_39', DOUBLE(asdecimal=False)) # 经营活动产生的现金流量净额
+    CASHFLOWSTATEMENT_59 = Column('cashflow_statement_59', DOUBLE(asdecimal=False)) # 投资活动产生的现金流量净额
+    CASHFLOWSTATEMENT_77 = Column('cashflow_statement_77', DOUBLE(asdecimal=False)) # 筹资活动产生的现金流量净额
+    CASHFLOWSTATEMENT_82 = Column('cashflow_statement_82', DOUBLE(asdecimal=False)) # 现金及现金等价物净增加额
+    CASHFLOWSTATEMENT_86 = Column('cashflow_statement_86', DOUBLE(asdecimal=False)) # 资产减值准备
+
+class EmStockRefinancing(Base):
+    '''Choice股票再融资信息表'''
+
+    __tablename__ = 'em_stock_refinancing'
+
+    id = Column(Integer, primary_key=True)
+
+    SECURITYCODE = Column('stock_id', CHAR(10), nullable=False) # EM股票id
+    APPROVENOTICEDATE = Column('approve_notice_date', DATE, nullable=False) # 最新公告日
+    PLANNOTICEDDATE = Column('plan_noticed_date', DATE, nullable=False) # 首次公告日
+    NEWPROGRESS = Column('new_progress', TEXT) # 方案进度
+    SUMFINA_T = Column('sum_fina_t', DOUBLE(asdecimal=False)) # 预计募资_上限(亿元)
+    ATTACHNAME = Column('attach_name', TEXT) # 原始公告链接
+    ADDPURPOSE = Column('add_purpose', TEXT, nullable=False) # 增发目的
 
 class EmIndexVal(Base):
     '''Choice指数估值数据'''
@@ -543,4 +628,9 @@ class EmIndexVal(Base):
     PCFTTM = Column("pcf_ttm", DOUBLE(asdecimal=False)) # 市现率PCF(TTM)
     ROE = Column("roe", DOUBLE(asdecimal=False)) # 净资产收益率
     EPSTTM = Column("eps_ttm", DOUBLE(asdecimal=False)) # 每股收益TTM
-    UPDATE_TIME = Column("update_time", DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP')) # 更新时间
+
+class EmTradeDates(Base):
+    '''Choice交易日数据'''
+
+    __tablename__ = 'em_tradedates'
+    TRADEDATES = Column(DATE, primary_key=True)
