@@ -40,6 +40,8 @@ class AssetPainter(object):
         l = [k for k, v in saa_weight.items() if v > 0]
         l.remove('cash')
         df = index_price.copy()[l].loc[mdd_date1:mdd_date2].fillna(method='bfill')
+        if df.empty:
+            return 
         df = df / df.iloc[0]
         df.plot.line(figsize=(18,9),legend=False,fontsize = 13)
         l = pl.legend(loc='lower left',fontsize = 13)
@@ -102,12 +104,7 @@ class AssetPainter(object):
 
         pct_col = index_id + '_pct'
         price_col = index_id + '_price'
-        pct_df = index_pct[['hs300','csi500','gem','sp500rmb']].rename(columns= {
-                                                                    'hs300':'hs300_pct',
-                                                                    'csi500':'csi500_pct',
-                                                                    'gem':'gem_pct',
-                                                                    'sp500rmb':'sp500rmb_pct',
-                                                                    })
+        pct_df = index_pct[[index_id]].rename(columns= {index_id:f'{index_id}_pct'})
         price_df = index_price[['hs300','csi500','gem','sp500rmb']].rename(columns= {   
                                                                             'hs300':'hs300_price',
                                                                             'csi500':'csi500_price',
@@ -190,4 +187,20 @@ class AssetPainter(object):
                 _plot_dif(ax, high_i, df, False)
             for low_i in low_plus_periods:
                 _plot_dif(ax, low_i, df, True)
-            plt.show()
\ No newline at end of file
+            plt.show()
+
+    @staticmethod
+    def plot_saa_rebalance_on_mv(asset_mv:pd.DataFrame,
+                                rebalance_date:list,
+                                MinActionAmtDiff:float,
+                                annual_ret:float,
+                                mdd:float,
+                                ret_over_mdd:float):
+        p = asset_mv['mv'].plot.line(figsize=(12,9), color = 'dodgerblue')
+        m = MinActionAmtDiff
+        annual_ret = round(annual_ret,4)
+        mdd = round(mdd,4)
+        ret_over_mdd=round(ret_over_mdd,4)
+        t = plt.title(f'market value with AmtDiff {m} annual_ret {annual_ret} mdd {mdd} ret_over_mdd {ret_over_mdd} ', fontsize=20)
+        for d, r in asset_mv.loc[rebalance_date,:].iterrows():
+            pl.scatter(x=d, y=r.mv, c='r',s=50)
